@@ -21,9 +21,9 @@
  * @param data		(unused)
  */
 static void displayArgs(poptContext con,
-		/*@unused@*/ enum poptCallbackReason foo,
+		/*@unused@*/ enum poptCallbackReason UNUSED(foo),
 		struct poptOption * key, 
-		/*@unused@*/ const char * arg, /*@unused@*/ void * data)
+		/*@unused@*/ const char * UNUSED(arg), /*@unused@*/ void * UNUSED(data))
 	/*@globals fileSystem@*/
 	/*@modifies fileSystem@*/
 {
@@ -67,7 +67,7 @@ struct poptOption poptHelpOptions[] = {
 /**
  * @param table		option(s)
  */
-/*@observer@*/ /*@null@*/ static const char *
+/*@observer@*/ /*@null@*/ static const char *const
 getTableTranslationDomain(/*@null@*/ const struct poptOption *table)
 	/*@*/
 {
@@ -85,10 +85,10 @@ getTableTranslationDomain(/*@null@*/ const struct poptOption *table)
  * @param opt		option(s)
  * @param translation_domain	translation domain
  */
-/*@observer@*/ /*@null@*/ static const char *
+/*@observer@*/ /*@null@*/ static const char *const
 getArgDescrip(const struct poptOption * opt,
 		/*@-paramuse@*/ /* FIX: i18n macros disabled with lclint */
-		/*@null@*/ const char * translation_domain)
+		/*@null@*/ const char * MAYBE_UNUSED(translation_domain))
 		/*@=paramuse@*/
 	/*@*/
 {
@@ -126,7 +126,7 @@ static /*@only@*/ /*@null@*/ char *
 singleOptionDefaultValue(int lineLength,
 		const struct poptOption * opt,
 		/*@-paramuse@*/ /* FIX: i18n macros disabled with lclint */
-		/*@null@*/ const char * translation_domain)
+		/*@null@*/ const char * MAYBE_UNUSED(translation_domain))
 		/*@=paramuse@*/
 	/*@*/
 {
@@ -196,7 +196,7 @@ singleOptionDefaultValue(int lineLength,
  */
 static void singleOptionHelp(FILE * fp, int maxLeftCol, 
 		const struct poptOption * opt,
-		/*@null@*/ const char * translation_domain)
+		/*@null@*/ const char * MAYBE_UNUSED(translation_domain))
 	/*@globals fileSystem @*/
 	/*@modifies *fp, fileSystem @*/
 {
@@ -332,7 +332,7 @@ static void singleOptionHelp(FILE * fp, int maxLeftCol,
 /*@-boundsread@*/
     while (helpLength > lineLength) {
 	const char * ch;
-	char format[30];
+	char format[16];
 
 	ch = help + lineLength - 1;
 	while (ch > help && !isspace(*ch)) ch--;
@@ -442,7 +442,7 @@ static void singleTableHelp(poptContext con, FILE * fp,
 	/*@modifies *fp, fileSystem @*/
 {
     const struct poptOption * opt;
-    const char *sub_transdom;
+    const char * sub_transdom;
 
     if (table == poptAliasOptions) {
 	itemHelp(fp, con->aliases, con->numAliases, left, NULL);
@@ -499,7 +499,7 @@ static int showHelpIntro(poptContext con, FILE * fp)
     return len;
 }
 
-void poptPrintHelp(poptContext con, FILE * fp, /*@unused@*/ int flags)
+void poptPrintHelp(poptContext con, FILE * fp, /*@unused@*/ int UNUSED(flags))
 {
     int leftColWidth;
 
@@ -678,13 +678,12 @@ static int showShortOptions(const struct poptOption * opt, FILE * fp,
 	/*@globals fileSystem @*/
 	/*@modifies *str, *fp, fileSystem @*/
 {
-    size_t size = 300;	/* larger then the ascii set */
-    char * s = alloca(size);
+    char * s = alloca(300);	/* larger then the ascii set */
 
     s[0] = '\0';
     /*@-branchstate@*/		/* FIX: W2DO? */
     if (str == NULL) {
-	memset(s, 0, size);
+	memset(s, 0, sizeof(s));
 	str = s;
     }
     /*@=branchstate@*/
@@ -707,7 +706,7 @@ static int showShortOptions(const struct poptOption * opt, FILE * fp,
     return strlen(s) + 4;
 }
 
-void poptPrintUsage(poptContext con, FILE * fp, /*@unused@*/ int flags)
+void poptPrintUsage(poptContext con, FILE * fp, /*@unused@*/ int UNUSED(flags))
 {
     poptDone done = memset(alloca(sizeof(*done)), 0, sizeof(*done));
     int cursor;
