@@ -2,7 +2,8 @@
  * 
  * distcc -- A simple distributed compiler system
  *
- * Copyright (C) 2002, 2003, 2004 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 2003, 2005 by Apple Computer, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,10 +21,11 @@
  * USA
  */
 
-                    /* Near is thy forgetfulness of all things; and near the
-                     * forgetfulness of thee by all.
-                     *                 -- Marcus Aurelius
-                     */
+
+		/* I don't know that atheists should be considered as
+		 * citizens, nor should they be considered patriots.
+		 * This is one nation under God.
+		 * -- Vice President, George H. W. Bush, 1987 */
 
 
 /**
@@ -69,24 +71,23 @@
 #include "exec.h"
 #include "tempfile.h"
 #include "srvnet.h"
-#include "types.h"
-#include "daemon.h"
-#include "netutil.h"
+#include "indirect_server.h"
+#include "indirect_util.h"
+#include "zeroconf_reg.h"
+#include "zeroconf_util.h"
 
-static int dcc_start_server_child(int listen_fd, int accept_fd, struct sockaddr *,
-                                  int);
-static void dcc_nofork_parent(int listen_fd) NORETURN;
-static void dcc_forking_parent(int listen_fd) NORETURN;
-static void dcc_detach(void);
+#ifndef WAIT_ANY
+#  define WAIT_ANY (-1)
+#endif
+
+static int dcc_start_child(int listen_fd, int);
+static void dcc_parent_loop(int listen_fd) NORETURN;
+static int dcc_serve_connection(int, int acc_fd);
+void dcc_detach(void);
 static void dcc_save_pid(pid_t);
-int dcc_nkids = 0;
+void dcc_reap_kids(void);
 
-
-/**
- * In forking or prefork mode, the maximum number of connections we want to
- * allow at any time.
- **/
-int dcc_max_kids = 0;
+static int nkids = 0;
 
 
 /**
@@ -98,79 +99,44 @@ int dcc_standalone_server(void)
 {
     int listen_fd;
     int n_cpus;
-    int ret;
+#if defined(DARWIN)
+    char *zcTxtRecord;
+#endif // DARWIN
 
-    if ((ret = dcc_socket_listen(arg_port, &listen_fd, opt_listen_addr)) != 0)
-        return ret;
+    if ((listen_fd = open_socket_in(arg_port)) == -1)
+        return EXIT_BIND_FAILED;
 
-    dcc_defer_accept(listen_fd);
-    
     set_cloexec_flag(listen_fd, 1);
 
-    if (dcc_ncpus(&n_cpus) == 0) 
-        rs_log_info("%d CPU%s online on this server", n_cpus, n_cpus == 1 ? "" : "s");
+    rs_log(RS_LOG_INFO|RS_LOG_NONAME,
+           "distccd (version %s, built %s %s) listening on port %d",
+           PACKAGE_VERSION, __DATE__, __TIME__, arg_port);
 
-    /* By default, allow one job per CPU, plus two for the pot.  The extra
-     * ones are started to allow for a bit of extra concurrency so that the
-     * machine is not idle waiting for disk or network IO. */
-    if (arg_max_jobs)
-        dcc_max_kids = arg_max_jobs;
-    else
-        dcc_max_kids = 2 + n_cpus;
-    
-    rs_log_info("allowing up to %d active jobs", dcc_max_kids);
+    if (dcc_ncpus(&n_cpus) == 0) 
+        rs_log_info("%d CPU%s online", n_cpus, n_cpus == 1 ? "" : "s");
+
+    dcc_catch_signals();
+
+#if defined(DARWIN)
+    zcTxtRecord = dcc_generate_txt_record();
+#endif
 
     if (!opt_no_detach) {
         /* Don't go into the background until we're listening and
          * ready.  This is useful for testing -- when the daemon
          * detaches, we know we can go ahead and try to connect.  */
         dcc_detach();
+	chdir("/");
     } else {
-        /* Still create a new process group, even if not detached */
-        rs_trace("not detaching");
-        if ((ret = dcc_new_pgrp()) != 0)
-            return ret;
         dcc_save_pid(getpid());
     }
-
-    /* Don't catch signals until we've detached or created a process group. */
-    dcc_daemon_catch_signals();
-
-    /* This is called in the master daemon, whether that is detached or
-     * not.  */
-    dcc_master_pid = getpid();
-
-    if (opt_no_fork) {
-        dcc_log_daemon_started("non-forking daemon");   
-        dcc_nofork_parent(listen_fd);
-    } else if (opt_no_prefork) {
-        dcc_log_daemon_started("forking daemon");
-        dcc_forking_parent(listen_fd);		/* noreturn */
-    } else {
-        dcc_log_daemon_started("preforking daemon");
-        return dcc_preforking_parent(listen_fd);
-    }
-}
-
-
-
-static void dcc_log_child_exited(pid_t kid,
-                                 int status)
-{
-    if (WIFSIGNALED(status)) {
-        int sig = WTERMSIG(status);
-        int severity = sig == SIGTERM ? RS_LOG_INFO : RS_LOG_ERR;
-        
-#ifdef HAVE_STRSIGNAL
-        /* take off every sig! for great justice. */
-        rs_log(severity, "child %d: %s", (int) kid, strsignal(sig));
-#else
-        rs_log(severity, "child %d: signal %d", (int) kid, sig);
+ 
+#if defined(DARWIN)
+    // Spawn a thread to handle the ZeroConfig task for this daemon.
+    dcc_register_for_zeroconfig(zcTxtRecord);
 #endif
-    } else if (WIFEXITED(status)) {
-        rs_log_info("child %d exited: exit status %d",
-                    (int) kid, WEXITSTATUS(status));
-    }
+
+    dcc_parent_loop(listen_fd);		/* noreturn */
 }
 
 
@@ -178,30 +144,28 @@ static void dcc_log_child_exited(pid_t kid,
 /**
  * @sa dcc_wait_child(), which is used by a process that wants to do a blocking
  * wait for some task like cpp or gcc.
- *
- * @param must_reap If True, don't return until at least one child has been
- * collected.  Used when e.g. all our process slots are full.  In either case
- * we keep going until all outstanding zombies are collected.
- *
- * FIXME: Are blocking waits meant to collect all of them, or just one?  At
- * the moment it waits until all children have exited.
  **/
-void dcc_reap_kids(int must_reap)
+void dcc_reap_kids(void)
 {
     while (1) {
         int status;
         pid_t kid;
 
-        kid = waitpid(WAIT_ANY, &status, must_reap ? 0 : WNOHANG);
+        kid = waitpid(WAIT_ANY, &status, WNOHANG);
         if (kid == 0) {
-            /* nobody has exited */
             break;
         } else if (kid != -1) {
             /* child exited */
-            --dcc_nkids;
-            rs_trace("down to %d children", dcc_nkids);
-
-            dcc_log_child_exited(kid, status);
+            --nkids;
+            rs_log_info("down to %d children", nkids);
+            
+            if (WIFSIGNALED(status)) {
+                rs_log_error("child %d exited on signal %d",
+                             (int) kid, WTERMSIG(status));
+            } else if (WIFEXITED(status)) {
+                rs_log_notice("child %d exited: exit status %d",
+                              (int) kid, WEXITSTATUS(status));
+            }
         } else if (errno == ECHILD) {
             /* No children left?  That's ok, we'll go back to waiting
              * for new connections. */
@@ -214,124 +178,95 @@ void dcc_reap_kids(int must_reap)
             continue;       /* loop again */
         } else {
             rs_log_error("wait failed: %s", strerror(errno));
-            /* e.g. too many open files; nothing we can do */
             dcc_exit(EXIT_DISTCC_FAILED);
         }
-
-        /* If there are more children keep looking, but don't block once we've
-         * collected at least one. */
-        must_reap = FALSE;
     }
 }
 
 
 /**
- * Main loop for no-fork mode.
- *
- * Much slower and may leak.  Should only be used when you want to run gdb on
- * distccd.
+ * Main loop for the parent process.  Basically it does nothing but
+ * wait around to be signalled, or for its children to die.  What a
+ * life!
  **/
-static void dcc_nofork_parent(int listen_fd)
+static void dcc_parent_loop(int listen_fd)
 {
     while (1) {
         int acc_fd;
-        struct dcc_sockaddr_storage cli_addr;
-        socklen_t cli_len;
 
         rs_log_info("waiting to accept connection");
-
-        cli_len = sizeof cli_addr;
-        acc_fd = accept(listen_fd,
-                        (struct sockaddr *) &cli_addr, &cli_len);
-        if (acc_fd == -1 && errno == EINTR) {
-            ; 
-        }  else if (acc_fd == -1) {
-            rs_log_error("accept failed: %s", strerror(errno));
-            dcc_exit(EXIT_CONNECT_FAILED);
-        } else {
-            dcc_service_job(acc_fd, acc_fd, (struct sockaddr *) &cli_addr, cli_len);
-            dcc_cleanup_tempfiles();
-            dcc_close(acc_fd);
-        }
-    }
-}
-
-
-/**
- * Main loop for the parent process, with the old behaviour of forking as
- * connections are accepted.  Basically it does nothing but wait around to be
- * signalled, or for its children to die.  What a life!
- **/
-static void dcc_forking_parent(int listen_fd)
-{
-    while (1) {
-        struct dcc_sockaddr_storage cli_addr;
-        socklen_t cli_len;
-        int acc_fd;
-
-        dcc_reap_kids(FALSE);
-
-        if (dcc_nkids >= dcc_max_kids) {
-            rs_trace("too busy for new connections, waiting for child to complete");
-            dcc_reap_kids(TRUE);
-        }
-
-        rs_log_info("waiting to accept connection");
-
-        cli_len = sizeof cli_addr;
-        acc_fd = accept(listen_fd, (struct sockaddr *) &cli_addr, &cli_len);
+        
+        acc_fd = accept(listen_fd, NULL, 0);
         if (acc_fd == -1 && errno == EINTR) {
             /* just go ahead and check for children */
         }  else if (acc_fd == -1) {
             rs_log_error("accept failed: %s", strerror(errno));
             dcc_exit(EXIT_CONNECT_FAILED);
         } else {
-            dcc_start_server_child(listen_fd, acc_fd,
-                                   (struct sockaddr *) &cli_addr, cli_len);
+            dcc_serve_connection(listen_fd, acc_fd);
+        }
+
+        if (!opt_no_fork) {
+            /* Don't call if in no-fork mode, because that would
+             * incorrectly pick up compilers etc. */
+            dcc_reap_kids();
         }
     }
 }
 
+/**
+ * Called in the daemon parent when a connection is received.  Either
+ * forks a new child, or handles it in the process (debug mode), as
+ * appropriate.
+ **/
+static int dcc_serve_connection(int listen_fd, int acc_fd)
+{
+    int ret = 0;
+    
+    if (opt_no_fork) {
+        ret = dcc_accept_job(acc_fd);
+        dcc_cleanup_tempfiles();
+    } else {
+        dcc_start_child(listen_fd, acc_fd);
+        ++nkids;
+        rs_log_info("up to %d children", nkids);
+    }
+    if (close(acc_fd)) {          /* in parent */
+        rs_log_error("failed to close accepted fd%d: %s", acc_fd,
+                     strerror(errno));
+    }
 
-
+    return ret;
+}
 
 /**
- * Fork a child to handle an already-accepted incoming connecting.  Not called
- * when in opt_no_fork mode.
+ * Fork a child to handle an incoming job.  Not called when in
+ * opt_no_fork mode.
  **/
-static int dcc_start_server_child(int listen_fd, int accepted_fd,
-                                  struct sockaddr *cli_addr,
-                                  int cli_len)
+static int dcc_start_child(int listen_fd, int accepted_fd)
 {
     pid_t kid;
     int ret, close_ret;
     
     kid = fork();
     if (kid == 0) {
-        dcc_close(listen_fd);
-        ret = dcc_service_job(accepted_fd, accepted_fd, cli_addr, cli_len);
+        close(listen_fd);
+#if defined(DARWIN)
+        dcc_support_indirection(&accepted_fd);
+#endif // DARWIN
+        ret = dcc_accept_job(accepted_fd);
         close_ret = dcc_close(accepted_fd);
         dcc_exit(ret ? ret : close_ret);
         /* doesn't return */
     } else if (kid == -1) {
         rs_log_error("fork failed: %s", strerror(errno));
-        return EXIT_OUT_OF_MEMORY; /* probably */
-    } else {
-        /* in parent */
-        ++dcc_nkids;
-        rs_log_info("up to %d children", dcc_nkids);
-
-        if (close(accepted_fd)) {          /* in parent */
-            rs_log_error("failed to close accepted fd%d in parent: %s",
-                         accepted_fd, strerror(errno));
-        }
+        return -1;
     }
 
     return 0;
 }
 
 
-
 /**
  * Save the pid of the child process into the pid file, if any.
  *
@@ -390,11 +325,10 @@ void dcc_remove_pid(void)
  *
  * This function returns in the child, but not in the parent.
  **/
-static void dcc_detach(void)
+void dcc_detach(void)
 {
     int i;
     pid_t pid;
-    pid_t sid;
     
     dcc_ignore_sighup();
 
@@ -413,12 +347,8 @@ static void dcc_detach(void)
 
     /* detach from the terminal */
 #ifdef HAVE_SETSID
-    if ((sid = setsid()) == -1) {
-        rs_log_error("setsid failed: %s", strerror(errno));
-    } else {
-        rs_trace("setsid to session %d", (int) sid);
-    }
-#else /* no HAVE_SETSID */
+        setsid();
+#else
 #ifdef TIOCNOTTY
     i = open("/dev/tty", O_RDWR);
     if (i >= 0) {
@@ -426,8 +356,7 @@ static void dcc_detach(void)
         close(i);
     }
 #endif /* TIOCNOTTY */
-#endif /* not HAVE_SETSID */
-    
+#endif
     /* make sure that stdin, stdout an stderr don't stuff things
        up (library functions, for example) */
     for (i=0;i<3;i++) {
@@ -435,7 +364,9 @@ static void dcc_detach(void)
         open("/dev/null", O_RDWR);
     }
 
+#if ! defined(DARWIN)
     /* If there's a lifetime limit on this server (for testing) then it needs
      * to apply after detaching as well. */
     dcc_set_lifetime();
+#endif // ! DARWIN
 }
