@@ -25,12 +25,4 @@ int dcc_lock_host(const char *lockname,
                   int *lock_fd);
 
 int dcc_unlock(int lock_fd);
-
-int dcc_make_lock_filename(const char *lockname,
-                           const struct dcc_hostdef *host,
-                           int iter,
-                           char **);
-
-int dcc_open_lockfile(const char *fname, int *plockfd);
-
-
+int sys_lock(int fd, int block);
